@@ -30,6 +30,7 @@
 #include "Common.h"
 #include "FilterParameters/AbstractParameter.h"
 #include "FilterParameters/PointParameter.h"
+#include "Logger.h"
 #include "Misc.h"
 
 namespace GmicQt
@@ -188,6 +189,7 @@ bool FilterParametersWidget::build(const QString & name, const QString & hash, c
   while (it != _parameters.end()) {
     AbstractParameter * parameter = *it;
     if (parameter->addTo(this, row)) {
+      parameter->hideWidgets();
       grid->setRowStretch(row, 0);
       ++row;
     }
@@ -290,7 +292,7 @@ void FilterParametersWidget::setValues(const QStringList & list, bool notify)
     return;
   }
   if (_actualParametersCount != list.size()) {
-    TRACE << "Wrong number of values" << list << "expecting" << _actualParametersCount;
+    Logger::warning(QString("Parameters/SetValues: Wrong number of values %1 (expecting %2)").arg(list.size()).arg(_actualParametersCount));
     return;
   }
   auto itValue = list.begin();
@@ -308,7 +310,7 @@ void FilterParametersWidget::setVisibilityStates(const QList<int> & states)
     return;
   }
   if (_actualParametersCount != states.size()) {
-    TRACE << "Wrong number of states" << states << "expecting" << _actualParametersCount;
+    Logger::warning(QString("Parameters/SetVisibilities: Wrong number of values %1 (expecting %2)").arg(states.size()).arg(_actualParametersCount));
     return;
   }
 
